import { createSlice, createAsyncThunk, PayloadAction } from "@reduxjs/toolkit";
import http from "../../lib/https";
import { toast } from "sonner";



interface StationDetails {
	id: number;
	stationName: string;
	registeredAddress: string;
	connectorType: string;
	power: string;
	price: string;
	isAvailable: boolean;
}

interface StationDetailState {
	stationDetails: StationDetails[];
	loading: boolean;
	error: string | null;
}

// Initial state
const initialState: StationDetailState = {
	stationDetails: [],
	loading: false,
	error: null,
};

// Fetch Manager List (Async Thunk)
export const stationDetailList = createAsyncThunk<
	StationDetails[],
	void,
	{ rejectValue: string }
>("fetchStationsDetails", async (_, { rejectWithValue }) => {
	try {
		const token = localStorage?.getItem("authToken");
		if (!token) throw new Error("No token found");

		const response = await http.get("/manager-station-details");
		if (!response.data?.data) throw new Error("Invalid API response");
		return response.data.data;
	} catch (error: any) {
		toast.error("Error Fetching Managers: " + error.message);
		return rejectWithValue(
			error?.response?.data?.message || "An error occurred"
		);
	}
});

// Create Manager (Async Thunk)
export const addStationDetails = createAsyncThunk<
	StationDetails,
	{
		connectorType: string;
		power: string;
		price: string;
		isAvailable: boolean;
	},
	{ rejectValue: string }
>("addManager", async (data, { rejectWithValue }) => {
	try {
		const response = await http.post(
			"create-manager-station-details",
			data
		);
		toast.success("Station details created successfully");
		return response.data?.data;
	} catch (error: any) {
		toast.error("Error creating manager: " + error.response?.data?.message);
		return rejectWithValue(
			error.response?.data?.message || "An error occurred"
		);
	}
});

// Update Manager (Async Thunk)
export const updateStationDetails = createAsyncThunk<
	StationDetails,
	{
		id: number;
		connectorType: string;
		power: string;
		price: string;
		isAvailable: boolean;
	},
	{ rejectValue: string }
>(
	"updateManagerStationDetails",
	async ({ id, ...managerStationData }, { rejectWithValue }) => {
		if (!id) {
			return rejectWithValue("Manager ID is required.");
		}
		try {
			const response = await http.put(
				`/update-manager-station-details/${id}`,
				{ ...managerStationData }
			);
			toast.success("Station Details updated successfully");
			return response?.data;
		} catch (error: any) {
			toast.error("Error updating manager: " + error.message);
			return rejectWithValue(
				error.response?.data?.message || "An error occurred"
			);
		}
	}
);

// Delete Manager (Async Thunk)
export const deleteStationDetails = createAsyncThunk<
	string,
	string,
	{ rejectValue: string }
>("deleteManager", async (id, { rejectWithValue }) => {
	try {
		await http.delete(`/delete-manager/${id}`);
		toast.success("Station details deleted successfully!");
		return id;
	} catch (error: any) {
		toast.error("Error deleting manager: " + error.message);
		return rejectWithValue(
			error.response?.data?.message || "An error occurred"
		);
	}
});

// Create Slice
const managerStationSlice = createSlice({
	name: "manager-station",
	initialState,
	reducers: {},
	extraReducers: (builder) => {
		builder
			// Fetch Managers
			.addCase(stationDetailList.pending, (state) => {
				state.loading = true;
				state.error = null;
			})
			.addCase(
				stationDetailList.fulfilled,
				(state, action: PayloadAction<StationDetails[]>) => {
					state.loading = false;
					state.stationDetails = action.payload;
				}
			)
			.addCase(stationDetailList.rejected, (state, action) => {
				state.loading = false;
				state.error = action.payload || "Failed to fetch managers";
			})

			// Add Station details
			.addCase(addStationDetails.pending, (state) => {
				state.loading = true;
			})
			.addCase(
				addStationDetails.fulfilled,
				(state, action: PayloadAction<StationDetails>) => {
					state.loading = false;
					state.stationDetails.push(action.payload);
				}
			)
			.addCase(addStationDetails.rejected, (state, action) => {
				state.loading = false;
				state.error = action.payload || "Failed to add manager";
			})

			// Update Manager
			.addCase(updateStationDetails.pending, (state) => {
				state.loading = true;
			})
			.addCase(updateStationDetails.fulfilled, (state, action) => {
				state.loading = false;
				const updateStationDetail = action.payload;
				const index = state.stationDetails.findIndex(
					(stationDetail) =>
						stationDetail.id === updateStationDetail.id
				);
				if (index !== -1) {
					state.stationDetails[index] = updateStationDetail; // Update the manager station in the state
				}
			})
			.addCase(updateStationDetails.rejected, (state, action) => {
				state.loading = false;
				state.error = action.payload || "Failed to update manager";
			})

			// Delete Manager
			.addCase(deleteStationDetails.pending, (state) => {
				state.loading = true;
			})
			.addCase(deleteStationDetails.fulfilled, (state, action) => {
				state.loading = false;
				state.stationDetails = state.stationDetails.filter(
					(stationDetail) =>
						String(stationDetail.id) !== String(action.payload)
				);
			})
			.addCase(deleteStationDetails.rejected, (state, action) => {
				state.loading = false;
				state.error = action.payload || "Failed to delete manager";
			});
	},
});

export default managerStationSlice.reducer;
